const fs = require("fs");
const path = require("path");
const https = require("https");
const exec = require("child_process").exec;

const name = process.argv[2];
const repo = process.argv[3] || "kbrsh/moon-template";
const archive = {
	method: "GET",
	host: "api.github.com",
	path: `/repos/${repo}/tarball/master`,
	headers: {
		"User-Agent": "Node.js"
	}
};

function log(type, message) {
	console.log(`\x1b[34m${type}\x1b[0m ${message}`);
}

function replace(content, sub, subNewString) {
	const index = content.indexOf(sub);

	if (index === -1) {
		return content;
	} else {
		const left = content.slice(0, index);
		const right = replace(content.slice(index + sub.length), sub, subNewString);
		const subNew = Buffer.from(subNewString);

		return Buffer.concat([left, subNew, right], left.length + subNew.length + right.length);
	}
}

function download(res) {
	const archivePath = path.join(__dirname, "moon-template.tar.gz");
	const stream = fs.createWriteStream(archivePath);

	res.on("data", (chunk) => {
		stream.write(chunk);
	});

	res.on("end", () => {
		stream.end();

		log("download", repo);
		install(archivePath);
	});
}

function install(archivePath) {
	const targetPath = path.join(process.cwd(), name);

	exec(`mkdir ${targetPath}`, (err) => {
		if (err) throw err;

		exec(`tar -xzf ${archivePath} -C ${targetPath} --strip=1`, (err) => {
			if (err) throw err;

			log("install", targetPath);
			clean(archivePath, targetPath);
		});
	});
}

function clean(archivePath, targetPath) {
	fs.unlink(archivePath, (err) => {
		if (err) throw err;

		log("clean", archivePath);
		create(targetPath, targetPath);
		log("success", `Generated project "${name}"`);
		console.log(`To start, run:
	cd ${name}
	npm install
	npm run dev`);
	});
}

function create(currentPath, targetPath) {
	const files = fs.readdirSync(currentPath);

	for (let i = 0; i < files.length; i++) {
		const file = files[i];
		const nextPath = path.join(currentPath, file);

		if (fs.statSync(nextPath).isDirectory()) {
			create(nextPath, targetPath);
		} else {
			fs.writeFileSync(nextPath, replace(fs.readFileSync(nextPath), "{# MoonName #}", name));
			log("create", path.relative(targetPath, nextPath));
		}
	}
}

log("Moon", "Generating project");

https.get(archive, (res) => {
	if (res.statusCode >= 300 && res.statusCode < 400 && res.headers.location !== undefined) {
		https.get(res.headers.location, (redirectRes) => {
			download(redirectRes);
		});
	} else {
		download(res);
	}
});
